using FITSIO, DelimitedFiles, Dierckx, Plots

function read_data2model_ratio(qdpfile::String)
    dmr=readdlm(qdpfile)
    nrows = length(dmr[:,1])
    println(nrows)
    data_break_row = Int64[]
    for i in 1:nrows
        if dmr[i,1] == "NO"
            push!(data_break_row,i)
            println(i)
        end
    end
    if length(data_break_row) == 1
        dmr1=dmr[4:data_break_row[1] - 1,1:4]
        en1=dmr1[:,1]
        rat1 = dmr1[:,3]
        dmr2=dmr[data_break_row[1] + 1 : nrows, 1:4]
        en2=dmr2[:,1]
        rat2 = dmr2[:,3]
        return en1,rat1, en2, rat2
    elseif length(data_break_row=0)
        dmr=dmr[4:dmr[end]-1,1:4]
        en=dmr[:,1]
        rat = dmr[:,3]
        return en, rat
    else
        println("More than two datasets, cannot handle at present!")
    end

end



function sxt_read_filt_trans(ftfile::String)
    f=FITS(ftfile)
    en_ev = read(f[2],"ENERGY")
    transmis = read(f[2],"TRANSMIS")
    en_kev = en_ev ./ 1000
    return Float64.(en_kev), Float64.(transmis)
end

function sxt_get_filt_trans_at_en(filt_en::Array{Float64,1}, filt_trans::Array{Float64,1}, en_new::Array{Float64,1})

    filt_trans_intrp = Spline1D(filt_en, filt_trans)
     filt_trans_new = filt_trans_intrp.(en_new)
     return en_new, filt_trans_new
end

function sxt_read_arf(arffile::String)
    f=FITS(arffile)
    elo = read(f[2],"ENERG_LO")
    ehi = read(f[2],"ENERG_HI")
    area = read(f[2],"SPECRESP")
    return elo,ehi, area
end

function sxt_arf_correct_filttrans(ftfile::String,oldarffile::String,newarffile::String)
    (en_ft, trans_ft) = sxt_read_filt_trans(ftfile)
    # Read arf
    f=FITS(oldarffile)
    header=read_header(f[2])
    elo=read(f[2],"ENERG_LO")
    ehi=read(f[2],"ENERG_HI")
    area = read(f[2],"SPECRESP")
    close(f)
    en_arf = (elo .+ ehi)/2
    (en_arf,ftrans) = sxt_get_filt_trans_at_en(en_ft,trans_ft,en_arf)
    plot(en_ft,trans_ft)
    plot!(en_arf,ftrans)
    area_corr_ft = area ./ ftrans
    fnew = FITS(newarffile,"w")
    colunits = Dict("ENERG_LO" => "keV", "ENERG_HI" => "keV", "SPECRESP" => "cm**2")
    write(fnew,["ENERG_LO", "ENERG_HI", "SPECRESP"], [elo,ehi,area_corr_ft], hdutype=TableHDU, header=header, name="SPECRESP",units=colunits)
    close(fnew)
    return newarffile
end


function sxt_arf_correct(oldarffile,newarffile)

    # Correct SXT arf based on the best-fitting function to data to model ratio

    f=FITS(oldarffile)
    header=read_header(f[2])
    elo=read(f[2],"ENERG_LO")
    ehi=read(f[2],"ENERG_HI")
    ea = read(f[2],"SPECRESP")
    close(f)
    # ARF correction
    en_kev = (elo .+ ehi) ./ 2

    # data to model ratio fit function
    p1c0=1.0
    p1c1=0.0055504
    g1fwhm = 0.938763
    g1pos  = 2.03697
    g1ampl  =  -0.122775
    g2fwhm  =  0.202793
    g2pos  =  0.818973
    g2ampl =  0.244481
    g3fwhm  =  0.610656
    g3pos   =   0.550279
    g3ampl   =     -0.225266
    g4fwhm    =    0.245633
    g4pos    =     1.84611
    g4ampl   =     0.151959
    g5fwhm   =     0.323751
    g5pos     =    2.21976
    g5ampl   =     0.205527

    g1(en_kev)= g1ampl * exp(-4 * log(2) * (en_kev - g1pos)^2 / g1fwhm^2)
    g2(en_kev)= g2ampl * exp(-4 * log(2) * (en_kev - g2pos)^2 / g2fwhm^2)
    g3(en_kev)= g3ampl * exp(-4 * log(2) * (en_kev - g3pos)^2 / g3fwhm^2)
    g4(en_kev)= g4ampl * exp(-4 * log(2) * (en_kev - g4pos)^2 / g4fwhm^2)
    g5(en_kev)= g5ampl * exp(-4 * log(2) * (en_kev - g5pos)^2 / g5fwhm^2)


    poly1(en_kev) = p1c0 + p1c1 * en_kev
    sxt_d2m_ratio_function(en_kev) = poly1(en_kev) + g1(en_kev) + g2(en_kev) + g3(en_kev) + g4(en_kev) + g5(en_kev)

# Correct effe area by multiplying original eff area by the data-to-model ratio

    ea_corr = ea .* sxt_d2m_ratio_function.(en_kev)
# Write New ARF fits file
    fnew = FITS(newarffile,"w")
    colunits = Dict("ENERG_LO" => "keV", "ENERG_HI" => "keV", "SPECRESP" => "cm**2")
    write(fnew,["ENERG_LO", "ENERG_HI", "SPECRESP"], [elo,ehi,ea_corr], hdutype=TableHDU, header=header, name="SPECRESP",units=colunits)
    close(fnew)

    return newarffile
end

function gabs(en_kev::Float64,line_en_kev::Float64, sigma_kev::Float64, line_depth::Float64)

    return exp(-(line_depth/(sqrt(2 * π * sigma_kev))) * exp.(-0.5*((en_kev - line_en_kev)/sigma_kev)^2))
end

function plabs(en_kev::Float64, α::Float64, K::Float64)
    return K* en_kev^(-α)
end

function edge(en_kev::Float64,edge_en_kev::Float64,depth::Float64)
    if en_kev < edge_en_kev
        return 1.0
    else
        return exp(-depth*(en_kev/edge_en_kev)^(-3))
    end
end

function gaussian_line(en_kev::Float64,line_en_kev::Float64, sigma_kev::Float64, line_norm::Float64)
    return line_norm * (1/(sigma_kev * sqrt(2 * π))) * exp(-(en_kev - line_en_kev)^2 / (2*sigma_kev^2))
end

function sxt_arf_correct_for_mult_component(oldarffile,newarffile)
# Correct SXT arf based on energy-dependent multiplicative factor in different bands

f=FITS(oldarffile)
header=read_header(f[2])
elo=read(f[2],"ENERG_LO")
ehi=read(f[2],"ENERG_HI")
ea = read(f[2],"SPECRESP")
close(f)
# ARF correction
en_kev = (elo .+ ehi) ./ 2
ea_corrected = Array{Float64}(undef, length(ea))
 ea_corrected .= ea

# Correction factor for 0.3-0.35keV
#elements_below_p35kev = [i  for  i in 1:length(en_kev) if en_kev[i] .<= 0.35]
#ea_corrected[elements_below_p35kev] .= ea[elements_below_p35kev]  .* 0.5


# Correction factor for 0.52-0.6keV
#elements_p52top6kev = [i  for  i in 1:length(en_kev) if en_kev[i] .> 0.52 && en_kev[i] .<= 0.6]
#ea_corrected[elements_p52top6kev] .= ea[elements_p52top6kev]  .* 1.2

# Correction in the 0.55-0.9keV bands
#elements_p55top9kev = [i  for  i in 1:length(en_kev) if en_kev[i] .> 0.55 && en_kev[i] .<= 0.9]
#ea_corrected[elements_p55top9kev] = ea[elements_p55top9kev] .* 1.04731 .* elements_p55top9kev .^0.364

#elements_p9top2kev = [i  for  i in 1:length(en_kev) if en_kev[i] .> 0.9 && en_kev[i] .<= 2.0]
#ea_corrected[elements_p9top2kev] = ea[elements_p9top2kev] .* 0.93 .* elements_p9top2kev .^0.18

# Correction above -0.9keV
#elements_above_2kev = [i  for  i in 1:length(en_kev) if en_kev[i] .> 2]
#ea_corrected[elements_above_2kev] = ea[elements_above_2kev] .* 1.1 .* elements_above_2kev.^0.054

ea_corrected = ea .* plabs.(en_kev,0.06,1.0)
#ea_corrected = ea .* exp.(-(0.02/(sqrt(2 * π * 0.0412))) .* exp.(-0.5.*((en_kev .- 0.404)./0.0412).^2))
#ea_corrected = ea .* 1.17257 .* en_kev.^-0.026



# Correction factor above 1 keV
#elements_below_1kev = [i  for  i in 1:length(en_kev) if en_kev[i] .<= 1.0]
#println(en_kev[elements_below_1kev])
#ea_corrected[elements_below_1kev] .=  ea[elements_below_1kev] .* (1.07 .* en_kev[elements_below_1kev] .^0.032)

#ea_corrected .= ea .* gabs.(en_kev, 0.496271, 0.123540, 0.145162) .* gabs.(en_kev, 0.931419, 0.421697, 0.453801) .* gabs.(en_kev, 1.97530, 1.13428, 1.20513) .* plabs.(en_kev, -0.3109, 1.0)

#elements_above_1kev = [i  for  i in 1:length(en_kev) if en_kev[i] .> 1.0]
#ea_corrected[elements_above_1kev] = ea[elements_above_1kev] .* 1.03

# for 3-4.5 keV
#elements_3to4p5kev = [i  for  i in 1:length(en_kev) if en_kev[i] .> 3.0 && en_kev[i] .<= 4.5]
#ea_corrected[elements_3to4p5kev] = ea[elements_3to4p5kev] .* plabs.(en_kev[elements_3to4p5kev], -0.33, 0.65)
#ea_corrected .= ea .* gabs.(en_kev, 0.0307632,0.313079, 0.697271)
#ea_corrected .= ea .* gabs.(en_kev, 0.524899, 0.0477979, 0.0774243) .* gabs.(en_kev, 1.73123, 1.62656, 1.05395)

fnew = FITS(newarffile,"w")
    colunits = Dict("ENERG_LO" => "keV", "ENERG_HI" => "keV", "SPECRESP" => "cm**2")
    write(fnew,["ENERG_LO", "ENERG_HI", "SPECRESP"], [elo,ehi,ea_corrected], hdutype=TableHDU, header=header, name="SPECRESP",units=colunits)
    close(fnew)
display(plot(en_kev,ea))
display(plot!(en_kev,ea_corrected))
return newarffile
end

function sxt_arf_correct_for_additive_component(oldarffile,newarffile)
# Correct SXT arf based on energy-dependent multiplicative factor in different bands

f=FITS(oldarffile)
header=read_header(f[2])
elo=read(f[2],"ENERG_LO")
ehi=read(f[2],"ENERG_HI")
ea = read(f[2],"SPECRESP")
close(f)
# ARF correction
en_kev = (elo .+ ehi) ./ 2
ea_corrected = Array{Float64}(undef, length(ea))
 ea_corrected .= ea

# Calculate new effective area
 ea_corrected .= ea .+ ea .* gaussian_line.(en_kev,2.13,0.083,5.5e-2)

 fnew = FITS(newarffile,"w")
     colunits = Dict("ENERG_LO" => "keV", "ENERG_HI" => "keV", "SPECRESP" => "cm**2")
     write(fnew,["ENERG_LO", "ENERG_HI", "SPECRESP"], [elo,ehi,ea_corrected], hdutype=TableHDU, header=header, name="SPECRESP",units=colunits)
     close(fnew)
 display(plot(en_kev,ea))
 display(plot!(en_kev,ea_corrected))
 return newarffile
 end
